==> ipymidicontrols-0.1.0.tar.gz: package/lib/xtouchmini/rotary.js <==
// Copyright (c) Project Jupyter Contributors
// Distributed under the terms of the Modified BSD License.
import midi from 'webmidi';
import { clamp } from './utils';
import { Signal } from '@phosphor/signaling';
import { Disposable } from './disposable';
export class Rotary extends Disposable {
    /**
     * control is the midi CC number.
     */
    constructor(control, { lightMode = 'single', min = 0, max = 100, value = 50 } = {}) {
        super();
        this._stateChanged = new Signal(this);
        this._control = control;
        this._lightMode = lightMode;
        this._min = min;
        this._max = max;
        midi.inputs[0].addListener('controlchange', 1, e => {
            if (e.controller.number === this._control) {
                // Value is relative
                let sign = e.value & 0x40 ? -1 : 1;
                let increment = sign * (e.value & 0x3f);
                this.value += increment;
            }
        });
        this.value = value;
    }
    refresh() {
        let factor = this._lightMode === 'spread' ? 6.999999 : 10.999999;
        //  assumes 11 leds around rotary, so convert value to between 1 and 11
        let leds = Math.trunc(((this._value - this._min) / (this._max - this._min)) * factor) + 1;
        midi.outputs[0].sendControlChange(0x20 + this._control, lightModeNums.get(this._lightMode) + leds, 1);
    }
    /**
     * value goes from min to max, inclusive.
     */
    get value() {
        return this._value;
    }
    set value(value) {
        const newValue = clamp(value, this._min, this._max);
        const oldValue = this._value;
        if (oldValue !== newValue) {
            this._value = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'value',
                oldValue,
                newValue
            });
        }
    }
    /**
     * Setting the min may bump the value and max if necessary.
     */
    get min() {
        return this._min;
    }
    set min(newValue) {
        const oldValue = this._min;
        if (oldValue !== newValue) {
            if (newValue > this.value) {
                this.value = newValue;
            }
            if (newValue > this.max) {
                this.max = newValue;
            }
            this._min = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'min',
                oldValue,
                newValue
            });
        }
    }
    /**
     * Setting the max may bump the value and min if necessary.
     */
    get max() {
        return this._max;
    }
    set max(newValue) {
        const oldValue = this._max;
        if (oldValue !== newValue) {
            if (newValue < this.value) {
                this.value = newValue;
            }
            if (newValue < this.min) {
                this.min = newValue;
            }
            this._max = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'max',
                oldValue,
                newValue
            });
        }
    }
    /**
     * values can be:
     * * 'single' - light up a single light on value
     * * 'trim' - light from current value to top
     * * 'fan' - light up from left to current value
     * * 'spread' - light up from top down both sides
     */
    get lightMode() {
        return this._lightMode;
    }
    set lightMode(newValue) {
        const oldValue = this._lightMode;
        if (oldValue !== newValue) {
            this._lightMode = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'lightMode',
                oldValue,
                newValue
            });
        }
    }
    /**
     * A signal fired when the widget state changes.
     */
    get stateChanged() {
        return this._stateChanged;
    }
}
/**
 * Map from the knob light mode to the appropriate MIDI command.
 */
const lightModeNums = new Map([
    ['single', 0],
    ['trim', 0x10],
    ['wrap', 0x20],
    ['spread', 0x30]
]);

==> ipymidicontrols-0.1.3.tar.gz: package/lib/xtouchmini/rotary.js <==
// Copyright (c) Project Jupyter Contributors
// Distributed under the terms of the Modified BSD License.
import midi from 'webmidi';
import { clamp } from './utils';
import { Signal } from '@phosphor/signaling';
import { Disposable } from './disposable';
export class Rotary extends Disposable {
    /**
     * control is the midi CC number.
     */
    constructor(control, { lightMode = 'single', min = 0, max = 100, value = 50 } = {}) {
        super();
        this._stateChanged = new Signal(this);
        this._control = control;
        this._lightMode = lightMode;
        this._min = min;
        this._max = max;
        const input = midi.inputs.find(x => x.manufacturer === "Behringer" && x.name.startsWith("X-TOUCH MINI"));
        if (!input) {
            throw new Error("Could not find Behringer X-TOUCH MINI");
        }
        input.addListener('controlchange', 1, e => {
            if (e.controller.number === this._control) {
                // Value is relative
                let sign = e.value & 0x40 ? -1 : 1;
                let increment = sign * (e.value & 0x3f);
                this.value += increment;
            }
        });
        this.value = value;
    }
    refresh() {
        let factor = this._lightMode === 'spread' ? 6.999999 : 10.999999;
        //  assumes 11 leds around rotary, so convert value to between 1 and 11
        let leds = Math.trunc(((this._value - this._min) / (this._max - this._min)) * factor) + 1;
        const output = midi.outputs.find(x => x.manufacturer === "Behringer" && x.name.startsWith("X-TOUCH MINI"));
        if (!output) {
            throw new Error("Could not find Behringer X-TOUCH MINI");
        }
        output.sendControlChange(0x20 + this._control, lightModeNums.get(this._lightMode) + leds, 1);
    }
    /**
     * value goes from min to max, inclusive.
     */
    get value() {
        return this._value;
    }
    set value(value) {
        const newValue = clamp(value, this._min, this._max);
        const oldValue = this._value;
        if (oldValue !== newValue) {
            this._value = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'value',
                oldValue,
                newValue
            });
        }
    }
    /**
     * Setting the min may bump the value and max if necessary.
     */
    get min() {
        return this._min;
    }
    set min(newValue) {
        const oldValue = this._min;
        if (oldValue !== newValue) {
            if (newValue > this.value) {
                this.value = newValue;
            }
            if (newValue > this.max) {
                this.max = newValue;
            }
            this._min = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'min',
                oldValue,
                newValue
            });
        }
    }
    /**
     * Setting the max may bump the value and min if necessary.
     */
    get max() {
        return this._max;
    }
    set max(newValue) {
        const oldValue = this._max;
        if (oldValue !== newValue) {
            if (newValue < this.value) {
                this.value = newValue;
            }
            if (newValue < this.min) {
                this.min = newValue;
            }
            this._max = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'max',
                oldValue,
                newValue
            });
        }
    }
    /**
     * values can be:
     * * 'single' - light up a single light on value
     * * 'trim' - light from current value to top
     * * 'fan' - light up from left to current value
     * * 'spread' - light up from top down both sides
     */
    get lightMode() {
        return this._lightMode;
    }
    set lightMode(newValue) {
        const oldValue = this._lightMode;
        if (oldValue !== newValue) {
            this._lightMode = newValue;
            this.refresh();
            this._stateChanged.emit({
                name: 'lightMode',
                oldValue,
                newValue
            });
        }
    }
    /**
     * A signal fired when the widget state changes.
     */
    get stateChanged() {
        return this._stateChanged;
    }
}
/**
 * Map from the knob light mode to the appropriate MIDI command.
 */
const lightModeNums = new Map([
    ['single', 0],
    ['trim', 0x10],
    ['wrap', 0x20],
    ['spread', 0x30]
]);
